"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const winston_1 = require("winston");
const DailyRotateFile = require("winston-daily-rotate-file");
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
const http_1 = require("./http");
const defauleLogEntries = [
    'timestamp',
    'level',
    'context',
    'message',
    'trace',
    'ip',
    'ua',
    'method',
    'url',
    'query',
    'body',
];
const levelPriorities = {
    error: 0,
    warn: 1,
    log: 2,
    debug: 3,
    verbose: 4,
};
function getWinstonLevel(level) {
    return level === 'log' ? 'info' : level;
}
function getNestLevel(level) {
    return level === 'info' ? 'log' : level;
}
function filterEmpty(obj) {
    return Object.keys(obj).reduce((map, k) => {
        const v = obj[k];
        if (typeof v === 'number' || !(0, lodash_1.isEmpty)(v)) {
            map[k] = v;
        }
        return map;
    }, {});
}
function getFormat(logEntries, fn) {
    return winston_1.format.combine(winston_1.format.timestamp({ format: () => +new Date() }), winston_1.format.printf((obj) => {
        obj.level = getNestLevel(obj.level);
        obj.ip = obj.ip || '::1';
        const payload = Object.assign(Object.assign({}, (0, lodash_1.pick)(obj, logEntries)), (0, lodash_1.omit)(obj, defauleLogEntries));
        return JSON.stringify(fn ? fn(payload) : payload);
    }));
}
class Logger {
    constructor(options, context) {
        this.context = context;
        this.level = 'verbose';
        this.logEntries = options.logEntries || defauleLogEntries;
        this.setLevel(options.level);
        this.loggers = {
            winston: (0, winston_1.createLogger)({
                transports: [
                    options.filename && options.dirname
                        ? new DailyRotateFile(Object.assign(Object.assign({ filename: options.filename, dirname: options.dirname, format: getFormat(this.logEntries) }, options), { level: 'silly' }))
                        : [],
                    options.http
                        ? new http_1.HttpTransport(Object.assign(Object.assign({ format: getFormat(this.logEntries, options.http.payload) }, options.http), { level: 'silly' }))
                        : [],
                ].flat(),
            }),
            nest: new common_1.ConsoleLogger(),
        };
    }
    static getContext(context, request) {
        return filterEmpty(Object.assign(Object.assign({}, (typeof context === 'object' ? context : { context })), { ip: request.ip, ua: request.get('user-agent'), method: request.method, url: request.originalUrl, query: request.query, body: request.body }));
    }
    all(message, level, context, trace) {
        context = Object.assign(Object.assign({}, this.context), (typeof context === 'object'
            ? context
            : { context: context }));
        if (levelPriorities[level] > levelPriorities[this.level]) {
            return;
        }
        this.loggers.nest[level](message.message || message, context.context);
        const winstonLevel = getWinstonLevel(level);
        if (message instanceof Error || message.message) {
            return this.loggers.winston.log(Object.assign({ level: winstonLevel, message: message.message, trace: [trace || message.stack] }, context));
        }
        return this.loggers.winston.log(Object.assign({ level: winstonLevel, message }, context));
    }
    setLevel(level) {
        if (!level || levelPriorities[level] === undefined)
            return;
        this.level = level;
    }
    error(message, trace, context) {
        this.all(message, 'error', context, trace);
    }
    log(message, context) {
        this.all(message, 'log', context);
    }
    warn(message, context) {
        this.all(message, 'warn', context);
    }
    debug(message, context) {
        this.all(message, 'debug', context);
    }
    verbose(message, context) {
        this.all(message, 'verbose', context);
    }
}
exports.Logger = Logger;
